package at.herold.test.objlifetime;

import java.lang.ref.PhantomReference;
import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Set;
/**
 * Class to monitor the lifetime of objects.
 * To monitor object creation, the classes to monitor must have
<code>
if (at.herold.test.objlifetime.ObjectCreationMonitoringFlag.monitoring)
{
   at.herold.test.objlifetime.ObjectCreationMonitoring.monitor(this);
}
</code>
 * in their constructor. To monitor ALL classes, 
 * <code>java.lang.Object</code> has to changed. Then the 
 * bootclasspath has to be PREPENDED with the classpath to the 
 * overriden-Object class.
 * @author Jack Shirazi, "Java Performance Tuning", p. 47
 * @author Peter Kofler
 */
public final class ObjectLifetimeMonitoring
{

   /**
    * Queue of the references to be enqueued before GC.
    */
   private static final ReferenceQueue queue = new ReferenceQueue();

   /**
    * List of all live references. 
    * This is needed because if the Reference is not reachable, it is 
    * not enqueued. Must be fast for searching. As the references use 
    * Object's hashcode, they are unique. So a set should be ok.
    */
   private static final Set references = new HashSet();

   /** Register threads. Must be done before starting of monitoring. */
   public static final synchronized void registerThreadsForMonitoring()
   {
      TimerThread.createAndStart();
      AdminThread.createAndStart();

      // register shutdown informer
      Runtime.getRuntime().addShutdownHook(new Thread()
      {
         public void run()
         {
            monitorShutdown();
         }
      });
   }

   /** 
    * Called by the modified constructor of <code>Object</code> to 
    * monitor a new object. This monitors the creation of the object 
    * and all pending enqueued references which will be collected.
    * @param o the instance of the new object.
    */
   public static final synchronized void monitor(Object o)
   {
      // disable object creation monitoring while we report
      MonitoringFlag.monitoring = false;

      Reference ref;
      while ((ref = queue.poll()) != null)
      {
         // System.out.println("LTM D @" + System.identityHashCode(ref) + " " + TimerThread.time);
         LifetimeStatistics.monitoredEvents.add(new MonitoredGCEvent(ref));
         references.remove(ref);
      }

      ref = new PhantomReference(o, queue);
      // System.out.println("LTM C @" + System.identityHashCode(ref) + " " + TimerThread.time);
      LifetimeStatistics.monitoredEvents.add(new MonitoredCreationEvent(ref, o.getClass()));
      references.add(ref);

      // re-enable object creation monitoring
      MonitoringFlag.monitoring = true;
   }

   static final synchronized void monitorShutdown()
   {
      // disable object creation monitoring while we report
      MonitoringFlag.monitoring = false;

      // System.out.println("LTM D @ALL " + TimerThread.time);
      LifetimeStatistics.saveMonitoredEvents(true);

      // re-enable object creation monitoring
      MonitoringFlag.monitoring = true;
   }

   public static final synchronized void saveMonitorStatistics()
   {
      // disable object creation monitoring while we report
      MonitoringFlag.monitoring = false;
      LifetimeStatistics.saveMonitoredEvents(false);
      // re-enable object creation monitoring
      MonitoringFlag.monitoring = true;
   }

   public static final synchronized void stopMonitoring()
   {
      MonitoringFlag.monitoring = false;
   }

   public static final synchronized void startMonitoring()
   {
      MonitoringFlag.monitoring = true;
   }

   /** Start the monitoring.
       @param args the parameters, classname and arguments. */
   public static void main(String[] args)
   {

      if (args.length < 2)
      {
         System.out.println("ObjectLifetimeMonitoring s|m <classname> [<parameters>]");
         System.out.println("starts the class' main with parameters.");
         System.out.println("m starts monitoring immediately, s just starts.");
         return;
      }

      try
      {
         final boolean start = args[0].equals("m");

         // remaining arguments are the class with the main() method, and its arguments
         final String classname = args[1];
         final String[] argz = new String[args.length - 2];
         System.arraycopy(args, 2, argz, 0, argz.length);
         // we need Systemclassloader; Bootstrap wil not load from user classpath.
         final Class clazz = ClassLoader.getSystemClassLoader().loadClass(classname);

         // main has one parameter, a String array.
         final Class[] mainParamType = { args.getClass()};
         final Method main = clazz.getMethod("main", mainParamType);
         final Object[] params = { argz };

         // start monitoring
         registerThreadsForMonitoring();
         if (start)
         {
            // sleep to give threads time to init before monitoring
            Thread.sleep(500);
            System.out.println("LTM started Monitoring and Application");
            startMonitoring();
         }
         else
         {
            System.out.println("LTM started Application");
         }

         // run programm
         main.invoke(null, params);

         stopMonitoring();
      }
      catch (Exception e)
      {
         System.out.println("LTM error in Application Starter");
         e.printStackTrace();
      }
   }

}