package at.herold.test.objlifetime.viewer;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
/**
 * Main class of the life time monitor viewers.
 * @author Peter Kofler
 */
public class Main
{

   private static final String IMAGE_SAVE_EXT = "jpg";

   private static final ViewedLifetimeStatistics statistics = new ViewedLifetimeStatistics();
   private static String fileName;

   public static void main(String[] args)
   {

      if (args.length < 1)
      {
         System.out.println("Usage: Main <life time statistics file>");
         System.out.println("loads, analyses and displays the object lifetime statistics.");
         System.out.println("Saves also the full image as " + IMAGE_SAVE_EXT + ".");
         return;
      }

      fileName = args[0];
      try
      {
         loadFile();

         analyseAndShowResults(new LifeObjectsPerTimeUnitAnalyser());

         analyseAndShowResults(new LifeObjectsPerClassAnalyser());

      }
      catch (Exception ex)
      {
         ex.printStackTrace();
      }

   }

   private static void analyseAndShowResults(final LifeTimeAnalyser analyser)
   {
      analyser.analyse(statistics);
      analyser.printResults();

      if (analyser.getPaintXAxis() > 0)
      {
         saveResultToImageFile(analyser);

         showResultsInFrame(analyser);
      }
   }

   private static void saveResultToImageFile(final LifeTimeAnalyser analyser)
   {
      final String imageFileName = fileName + "." + IMAGE_SAVE_EXT;
      System.out.println("saving "+analyser.getPaintXAxis()+"x"+analyser.getPaintYAxis()+" image "+imageFileName+"");

      final BufferedImage image = new BufferedImage(analyser.getPaintXAxis(), analyser.getPaintYAxis(), 1);
      final Graphics g = image.getGraphics();
      // have to clear it
      g.setColor(Color.WHITE);
      g.fillRect(0,0,analyser.getPaintXAxis(), analyser.getPaintYAxis());
      analyser.paintResults(g);
      try
      {
         final BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(imageFileName));
         final boolean success = ImageIO.write(image, IMAGE_SAVE_EXT, out);
         out.flush();
         out.close();

         if (!success)
         {
            System.out.println("could not save image - no image writer.");
         }
      }
      catch (IOException ex)
      {
         System.out.println("could not save image - " + ex.toString());
         ex.printStackTrace();
      }

   }

   private static void showResultsInFrame(final LifeTimeAnalyser analyser)
   {
      // show them
      final Canvas drawArea = new Canvas()
      {
         public void paint(Graphics g)
         {
            analyser.paintResults(g);
         }
      };
      drawArea.setSize(analyser.getPaintXAxis(), analyser.getPaintYAxis());
      drawArea.setBackground(Color.WHITE);

      JFrame frame = new JFrame(analyser.getClass().getName());
      frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      frame.getContentPane().add(drawArea, BorderLayout.CENTER);
      frame.pack();
      frame.setVisible(true);
   }

   private static void loadFile() throws IOException
   {
      System.out.println("Reading Lifetime File \"" + fileName + "\" - wait...");
      long time = System.currentTimeMillis();
      long mem = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();

      statistics.load(fileName);

      mem = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory() - mem;
      System.out.println("Read Memory: " + mem / 1024.0 + "kb");

      time = System.currentTimeMillis() - time;
      System.out.println("Read Time: " + time / 1000.0 + "s");

      System.out.println("Lines read " + statistics.getNumberLines());
   }

}
